/**
 * 
 */
package com.bukkitarena;

import java.io.File;
import java.io.IOException;

import org.bukkit.Bukkit;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.scheduler.BukkitScheduler;

import com.bukkitarena.arena.ArenaManager;
import com.bukkitarena.arenatype.ArenaTypeManager;
import com.bukkitarena.event.EventManager;
import com.bukkitarena.playerclass.ClassManager;
import com.bukkitarena.runnable.BAScheduler;
import com.bukkitarena.spawning.WaveManager;

/**
 * @author Nate Mortensen
 * 
 */
public class BukkitArena extends JavaPlugin {
    ArenaTypeManager typeManager;
    WaveManager waveManager;
    ClassManager classManager;
    ArenaManager arenaManager;
    VerboseLogger log;
    BAScheduler scheduler;
    static BukkitArena instance;
    EventManager eventManager;

    public void onEnable() {
        instance = this;
        // Create new objects.
        typeManager = new ArenaTypeManager();
        new Msg();
        waveManager = new WaveManager();
        classManager = new ClassManager();
        log = new VerboseLogger(getConfig().getString("logging"));
        scheduler = new BAScheduler();
        eventManager = new EventManager();

        startMetrics();

    }

    public void onDisable() {
        eventManager.clear();
        scheduler.clear();

        instance = null;
    }

    /**
	 * 
	 */
    public void startMetrics() {
        try {
            Metrics metrics = new Metrics(this);
            metrics.start();
        } catch (IOException e) {
            // Failed to submit the stats :-(
        }
    }

    public static EventManager getEventManager() {
        return instance.eventManager;
    }

    public static ArenaTypeManager getArenaTypeManager() {
        return instance.typeManager;
    }

    public static WaveManager getWaveManager() {
        return instance.waveManager;
    }

    public static ClassManager getClassManager() {
        return instance.classManager;
    }

    public static VerboseLogger getLog() {
        return instance.log;
    }

    public static ArenaManager getArenaManager() {
        return instance.arenaManager;
    }

    public static File getFolder() {
        return instance.getDataFolder();
    }

    // Hopefully allow arenas to use scheduled tasks.
    public static BAScheduler getScheduler() {
        return instance.scheduler;
    }

    public static BukkitScheduler getBukkitScheduler() {
        return Bukkit.getScheduler();
    }

    public static PluginManager getPluginManager() {
        return Bukkit.getPluginManager();
    }

    public static BukkitArena getInstance() {
        return instance;
    }

    /**
     * @return the TypeManager
     */
    public ArenaTypeManager getTYPEMANAGER() {
        return typeManager;
    }

    /**
     * @return the WaveManager
     */
    public WaveManager getWAVEMANAGER() {
        return waveManager;
    }

    /**
     * @return the ClassManager
     */
    public ClassManager getCLASSMANAGER() {
        return classManager;
    }

    /**
     * @return the ArenaManager
     */
    public ArenaManager getARENAMANAGER() {
        return arenaManager;
    }

    /**
     * @return the eVENTMANAGER
     */
    public EventManager getEVENTMANAGER() {
        return eventManager;
    }

}
