/**
 * 
 */
package com.bukkitarena.arena;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.bukkit.Location;
import org.bukkit.block.Block;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.Msg;
import com.bukkitarena.arenatype.ArenaType;
import com.bukkitarena.arenatype.ArenaTypeDescriptionFile;
import com.bukkitarena.arenatype.ArenaTypeManager;
import com.bukkitarena.arenatype.ArenaTypeSettings;
import com.bukkitarena.configuration.EasyConfiguration;
import com.bukkitarena.configuration.EasyManager;
import com.bukkitarena.entity.ArenaEntity;
import com.bukkitarena.entity.ArenaPlayer;
import com.bukkitarena.event.player.ArenaPlayerLeaveEvent;
import com.bukkitarena.objective.BlockObjective;
import com.bukkitarena.objective.RegionObjective;
import com.bukkitarena.playerclass.ClassSet;
import com.bukkitarena.region.Region;
import com.bukkitarena.spawning.Wave;
import com.bukkitarena.team.TeamColor;

/**
 * @author Nate Mortensen
 * 
 */
public class Arena extends EasyConfiguration {


    ArrayList<RegionObjective> regionObjectives = new ArrayList<RegionObjective>();

    ArrayList<BlockObjective> blockObjectives = new ArrayList<BlockObjective>();

    boolean running;

    ArenaType type;
    ConfigurationSection config;
    long startTime;
    ArenaTypeManager arenaTypeManager = BukkitArena.getArenaTypeManager();
    EasyManager<ArenaTypeSettings> atsm;
    ArenaSpawnpointManager asm;
    ClassSet classmanager;
    ArenaWaveManager wavemanager;
    Region region;
    PlayerManager pm;

    public Arena(ConfigurationSection c) {
        load(c);
        atsm = new EasyManager<ArenaTypeSettings>(ArenaTypeSettings.class, c);
        asm = new ArenaSpawnpointManager(config.getConfigurationSection("spawnpoints"));
        pm = new PlayerManager(this);

    }

    public PlayerManager getPlayerManager() {
        return pm;
    }

    public boolean isRunning() {
        return running;
    }

    public Region getRegion() {
        return region;
    }

    public void start(HashMap<Player, Location> p, String arenatype) {
        running = true;
        pm.players.addAll(wrapPlayers(p));
        startTime = System.currentTimeMillis();
        ArenaTypeSettings ats = atsm.get(arenatype);
        wavemanager = new ArenaWaveManager(ats.getWaveset(), this);
        classmanager = BukkitArena.getClassManager().getClassSet(ats.getClassset());
        type = arenaTypeManager.getNewArenaType(arenatype, this);
        type.onStart();
    }

    public List<String> canRun(String typename) {
        ArenaTypeDescriptionFile desc = arenaTypeManager.getDescriptionFile(typename);
        ArrayList<String> responses = new ArrayList<String>();
        if (desc.isNeedClassSet() && classmanager.isEmpty())
            responses.add("Type requires class set!");
        if (desc.isNeedWaveSet() && wavemanager.isEmpty())
            responses.add("Type requires wave set!");
        if (desc.isNeedSpawnpointSet() && asm.isEmpty())
            responses.add("Type requires spawnpoints!");
        if (!desc.isSatisfied(atsm.get(typename).getConfig(), this)) {
            responses.add("The following required values have not been set: ");
            for (String s : desc.getUnsatisfiedValues(atsm.get(typename).getConfig(), this))
                responses.add(s);
        }
        return responses;

    }

    public void end() {
        type.onEnd();
        BukkitArena.getEventManager().unregisterAll(type);
        type = null;
        startTime = 0;
        wavemanager = null;
        classmanager = null;
        running = false;

    }

    private List<ArenaPlayer> wrapPlayers(HashMap<Player, Location> p) {

        ArrayList<ArenaPlayer> ap = new ArrayList<ArenaPlayer>();
        for (Player player : p.keySet())
            ap.add(new ArenaPlayer(player, this, p.get(player)));
        return ap;
    }

    public long getStartTime() {
        return startTime;
    }

    public long getTimeSinceStart() {
        return (System.currentTimeMillis() - startTime);
    }

    public void setType(ArenaType t) {
        type = t;
    }

    public ArenaType getType() {
        return type;
    }

    public ConfigurationSection getConfig() {
        return config;
    }

    public void spawnTeam(TeamColor t) {
        // TODO redo this, using the new idea I had for entities, and replacing
        // them to make them completely customizable.
    }

    public void spawnWave(Wave w) {
        w.spawnWave(asm.spawnpoints);
    }

    public boolean isReady() {
        for (ArenaTypeSettings s : atsm)
            if (canRun(s.getName()).size() == 0)
                return true;
        return false;
    }

    public List<String> getTypes() {
        ArrayList<String> list = new ArrayList<String>();
        for (ArenaTypeSettings s : atsm)
            list.add(s.getName());
        return list;
    }

    public List<String> getRunnableTypes() {
        ArrayList<String> types = new ArrayList<String>();
        for (String s : getTypes())
            if (canRun(s).size() == 0)
                types.add(s);
        return types;
    }

    public ClassSet getClassSet() {
        return classmanager;
    }

    public ArenaSpawnpointManager getSpawnpointManager() {
        return this.asm;
    }

    public void spawnNextWave() {
        wavemanager.spawnNextWave();
    }

    public Wave getNextWave() {
        return wavemanager.getNextWave();
    }

    public ArenaWaveManager getWaveManager() {
        return wavemanager;
    }

    public void playerQuit(ArenaPlayer p) {
        ArenaPlayerLeaveEvent event = new ArenaPlayerLeaveEvent(p);
        BukkitArena.getEventManager().callEvent(event);
        if (event.isCancelled()) {
            p.sendMessage(Msg.ARENA_QUIT_FAIL);
            return;
        }
        pm.players.remove(p);
    }

    public BlockObjective getBlockObjective(Block b) {
        for (BlockObjective ob : blockObjectives)
            if (ob.contains(b))
                return ob;
        return null;
    }

    public RegionObjective getRegionObjective(Location l) {
        for (RegionObjective ro : regionObjectives)
            if (ro.containsLocation(l))
                return ro;
        return null;
    }

    public ArrayList<BlockObjective> getBlockObjectives() {
        return blockObjectives;
    }

    public ArrayList<RegionObjective> getRegionObjectives() {
        return regionObjectives;
    }

    public ArenaPlayer getPlayer(Player p) {
        return pm.getPlayer(p.getName());
    }

    public ArenaPlayer getPlayer(String name) {
        return pm.getPlayer(name);
    }

    public boolean containsPlayer(Player p) {
        return getPlayer(p) != null;
    }

    public ArenaEntity getEntity(LivingEntity e) {
        // TODO Finish the EntityManager.
        return null;
    }

    public void playerLeave(Player p) {
        // TODO
    }
}
