/**
 * 
 */
package com.bukkitarena.arena;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.EntityType;

import com.bukkitarena.spawning.Spawnpoint;
import com.bukkitarena.team.TeamColor;

/**
 * @author Nate Mortensen
 * 
 */
public class ArenaSpawnpointManager {
    ArrayList<Spawnpoint> spawnpoints;

    public ArenaSpawnpointManager(ArrayList<Spawnpoint> spawns) {
        spawnpoints = spawns;
    }

    public ArenaSpawnpointManager(ConfigurationSection s) {
        for (String spawn : s.getKeys(false)) {
            spawnpoints.add(new Spawnpoint(s.getConfigurationSection(spawn)));
        }
    }

    public void addSpawn(Spawnpoint point) {
        spawnpoints.add(point);
    }

    public void removeSpawn(Spawnpoint point) {
        if (spawnpoints.contains(point))
            spawnpoints.remove(point);
    }

    public boolean contains(String name) {
        for (Spawnpoint s : spawnpoints)
            if (s.getName().equalsIgnoreCase(name))
                return true;
        return false;

    }

    public Spawnpoint getRandomPoint() {
        return spawnpoints.get(new Random().nextInt(spawnpoints.size() - 1));
    }

    public Spawnpoint getRandomPoint(EntityType e) {
        if (!canSpawn(e)) {
            return null;
        }
        List<Spawnpoint> points = spawnpointsFor(e);
        return points.get(new Random().nextInt(points.size() - 1));
    }

    public Spawnpoint getRandomPoint(TeamColor t) {
        if (!canSpawn(t))
            return null;
        List<Spawnpoint> points = spawnpointsFor(t);
        return points.get(new Random().nextInt(points.size() - 1));
    }

    public ArrayList<Spawnpoint> spawnpointsFor(EntityType e) {
        ArrayList<Spawnpoint> points = new ArrayList<Spawnpoint>();
        for (Spawnpoint p : spawnpoints)
            if (p.canSpawn(e))
                points.add(p);
        return points;
    }

    public ArrayList<Spawnpoint> spawnpointsFor(TeamColor t) {
        ArrayList<Spawnpoint> points = new ArrayList<Spawnpoint>();
        for (Spawnpoint p : spawnpoints)
            if (p.canSpawn(t))
                points.add(p);
        return points;
    }

    public boolean canSpawn(EntityType e) {
        for (Spawnpoint p : spawnpoints) {
            if (p.canSpawn(e))
                return true;
        }
        return false;
    }

    public boolean canSpawn(TeamColor t) {
        for (Spawnpoint p : spawnpoints) {
            if (p.canSpawn(t))
                return true;
        }
        return false;
    }

    public boolean isEmpty() {
        return spawnpoints.isEmpty();
    }

    public List<Spawnpoint> getSpawnpoints() {
        return spawnpoints;
    }

}
