/**
 * 
 */
package com.bukkitarena.arena;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.bukkit.entity.Player;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.entity.ArenaPlayer;
import com.bukkitarena.event.player.ArenaPlayerQuitEvent;
import com.bukkitarena.team.TeamColor;
import com.bukkitarena.util.BoundedInteger;
import com.bukkitarena.util.SortedNamedObjectList;

/**
 * @author Nate Mortensen
 * 
 */
public class PlayerManager {
    SortedNamedObjectList<ArenaPlayer> players = new SortedNamedObjectList<ArenaPlayer>();

    SortedNamedObjectList<ArenaPlayer> spectators = new SortedNamedObjectList<ArenaPlayer>();

    Arena arena;

    public PlayerManager(Arena arena) {
        this.arena = arena;
    }

    public void reset() {
        for (ArenaPlayer p : players)
            p.sendHome();
        for (ArenaPlayer p : spectators)
            p.sendHome();
        clear();
    }

    public void clear() {
        players.clear();
        spectators.clear();
    }

    public void addPlayer(ArenaPlayer p) {
        players.add(p);
    }

    public void addSpectator(ArenaPlayer p) {
        spectators.add(p);
    }

    public void assignTeams(int numberOfTeams) {
        BoundedInteger index = new BoundedInteger(numberOfTeams);
        if (numberOfTeams > TeamColor.values().length)
            throw new IllegalArgumentException(Integer.toString(numberOfTeams) + " is more teams than physically possible.  This was done by Arena Type: " + arena.getType().getDescription().getName());
        TeamColor[] colors = TeamColor.values();
        for (ArenaPlayer player : players) {
            player.setTeam(colors[index.value()]);
        }

    }

    public ArenaPlayer getPlayer(Player p) {
        return getPlayer(p.getName());
    }

    public ArenaPlayer getPlayer(String name) {
        return players.get(name);
    }

    public boolean containsPlayer(Player p) {
        for (ArenaPlayer ap : players)
            if (ap.getPlayer().equals(p))
                return true;
        return false;
    }

    public void removePlayer(ArenaPlayer p) {
        ArenaPlayerQuitEvent quitEvent = new ArenaPlayerQuitEvent(p);
        BukkitArena.getEventManager().callEvent(quitEvent);
        p.sendHome();
        players.remove(p);
    }

    public void removePlayer(Player p) {
        removePlayer(getPlayer(p));
    }

    public SortedNamedObjectList<ArenaPlayer> getPlayers() {
        return players;
    }

    public SortedNamedObjectList<ArenaPlayer> getSpectators() {
        return spectators;
    }

}
