/**
 * 
 */
package com.bukkitarena.arenatype;

import java.io.InputStream;
import java.util.ArrayList;

import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.YamlConfiguration;

import com.bukkitarena.arena.Arena;
import com.bukkitarena.configuration.EasyConfiguration;
import com.bukkitarena.configuration.Load;

/**
 * @author Nate Mortensen
 * 
 */
public class ArenaTypeDescriptionFile extends EasyConfiguration {
    @Load
    String name;
    @Load
    String version;
    @Load
    String author;
    @Load
    String main;
    @Load
    boolean needClassSet;
    @Load
    boolean needSpawnpointSet;
    @Load
    boolean needWaveSet;
    @Load
    boolean pvp = true;
    @Load
    boolean allowBlockBreak;
    @Load
    boolean allowBlockPlace;
    ArrayList<String> registeredCommands = new ArrayList<String>();
    ArrayList<RequiredObject> requiredObjects = new ArrayList<RequiredObject>();
    // The next variables don't follow standard naming convention as to load
    // from the correct path;
    // These are used simply to check for valid spawn points. May remove these.
    @Load
    boolean spawn_player = false, spawn_mob = false;

    public ArenaTypeDescriptionFile(InputStream stream) {
        load(YamlConfiguration.loadConfiguration(stream));
        if (!getConfig().contains("requiredobjects"))
            return;
        for (String s : this.getConfig().getConfigurationSection("requiredobjects").getKeys(false)) {
            requiredObjects.add(new RequiredObject(getConfig().getConfigurationSection("requiredobjects." + s)));
        }
    }

    public boolean isSatisfied(ConfigurationSection c, Arena a) {
        return getUnsatisfiedValues(c, a).size() == 0;
    }

    public ArrayList<String> getUnsatisfiedValues(ConfigurationSection c, Arena a) {
        ArrayList<String> responses = new ArrayList<String>();
        for (RequiredObject obj : requiredObjects)
            if (!obj.isSatisfied(c, a)) {
                responses.add(obj.getName() + " (" + obj.type + ")");
                responses.add("- " + obj.getDescription());
            }
        return responses;
    }

    /**
     * @return the needClassSet
     */
    public boolean isNeedClassSet() {
        return needClassSet;
    }

    /**
     * @return the needSpawnpointSet
     */
    public boolean isNeedSpawnpointSet() {
        return needSpawnpointSet;
    }

    /**
     * @return the needWaveSet
     */
    public boolean isNeedWaveSet() {
        return needWaveSet;
    }

    /**
     * @return the spawn_player
     */
    public boolean isSpawn_player() {
        return spawn_player;
    }

    /**
     * @return the spawn_mob
     */
    public boolean isSpawn_mob() {
        return spawn_mob;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @return the version
     */
    public String getVersion() {
        return version;
    }

    /**
     * @return the author
     */
    public String getAuthor() {
        return author;
    }

    /**
     * @return the registeredCommands
     */
    public ArrayList<String> getRegisteredCommands() {
        return registeredCommands;
    }

    /**
     * @return the main
     */
    public String getMain() {
        return main;
    }

}
