/**
 * 
 */
package com.bukkitarena.arenatype;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.apache.commons.lang.Validate;
import org.yaml.snakeyaml.error.YAMLException;

/**
 * @author Nate Mortensen
 * 
 */
public class ArenaTypeLoader {
    public HashMap<ArenaTypeDescriptionFile, Class<? extends ArenaType>> loadTypes(File file) {
        HashMap<ArenaTypeDescriptionFile, Class<? extends ArenaType>> map = new HashMap<ArenaTypeDescriptionFile, Class<? extends ArenaType>>();
        for (File f : file.listFiles()) {
            if (!file.isDirectory() && file.getName().endsWith(".jar")) {
                ArenaTypeDescriptionFile description = getDescriptionFile(f);
                map.put(description, loadType(f, description));
            }
        }
        return map;

    }

    public Class<? extends ArenaType> loadType(File file, ArenaTypeDescriptionFile desc) {
        Validate.notNull(file, "File Cannot be null.");
        if (!file.exists() || !file.getAbsolutePath().endsWith(".jar") || file.isDirectory()) {
            throw new InvalidArenaTypeException("ArenaType must not be a directoy, and must be a .jar file.");
        }
        try {
            URL[] urls = new URL[1];
            urls[0] = file.toURI().toURL();
            Class<?> jarClass = Class.forName(desc.getMain(), true, new URLClassLoader(urls, this.getClass().getClassLoader()));
            Class<? extends ArenaType> type = jarClass.asSubclass(ArenaType.class);
            return type;

        } catch (Throwable e) {
            throw new InvalidArenaTypeException(e);

        }

    }

    public static ArenaTypeDescriptionFile getDescriptionFile(File file) {
        Validate.notNull(file, "File cannot be null.");
        JarFile jar = null;
        InputStream stream = null;
        try {
            jar = new JarFile(file);
            JarEntry entry = jar.getJarEntry("plugin.yml");
            if (entry == null) {
                throw new InvalidArenaTypeException(file.getName() + " is missing a plugin.yml");
            }
            stream = jar.getInputStream(entry);
            return new ArenaTypeDescriptionFile(stream);
        } catch (IOException ex) {
            throw new InvalidArenaTypeException("Invalid plguin.yml");
        } catch (YAMLException ex) {
            throw new InvalidArenaTypeException("Invalid plugin.yml");
        } finally {
            if (jar != null) {
                try {
                    jar.close();
                } catch (IOException e) {
                }
            }
            if (stream != null) {
                try {
                    stream.close();
                } catch (IOException e) {
                }
            }
        }

    }
}
