/**
 * 
 */
package com.bukkitarena.arenatype;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.arena.Arena;

/**
 * @author Nate Mortensen
 * 
 */
public class ArenaTypeManager {
    HashMap<ArenaTypeDescriptionFile, Class<? extends ArenaType>> types = new HashMap<ArenaTypeDescriptionFile, Class<? extends ArenaType>>();
    ArenaTypeLoader loader;

    public ArenaTypeManager() {
        loader = new ArenaTypeLoader();
        File typefolder = new File(BukkitArena.getFolder() + File.separator + "arenatypes" + File.separator);
        if (!typefolder.exists())
            try {
                typefolder.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        types = loader.loadTypes(typefolder);

    }

    public void reload() {
        File typefolder = new File(BukkitArena.getFolder(), "arenatypes" + File.separator);
        types = loader.loadTypes(typefolder);
    }

    public Class<? extends ArenaType> getType(String name) {
        for (ArenaTypeDescriptionFile f : types.keySet()) {
            if (f.getName().equalsIgnoreCase(name))
                return types.get(f);
        }
        return null;
    }

    public ArenaTypeDescriptionFile getDescriptionFile(String name) {
        for (ArenaTypeDescriptionFile f : types.keySet()) {
            if (f.getName().equalsIgnoreCase(name))
                return f;
        }
        return null;
    }

    public ArenaType getNewArenaType(String name, Arena a) {
        return getNewArenaType(getDescriptionFile(name), a);

    }

    public ArenaType getNewArenaType(ArenaTypeDescriptionFile f, Arena a) {
        try {
            Constructor<? extends ArenaType> constructor = types.get(f).getConstructor();
            ArenaType t = constructor.newInstance();
            t.initialize(a, this, f);
            return t;
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        }
        return null;
    }
}
