/**
 * 
 */
package com.bukkitarena.arenatype;

import org.bukkit.configuration.ConfigurationSection;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.arena.Arena;
import com.bukkitarena.configuration.EasyConfiguration;
import com.bukkitarena.configuration.Load;
import com.bukkitarena.util.ConfigUtil;

/**
 * @author Nate Mortensen
 * 
 */
public class RequiredObject extends EasyConfiguration {

	//TODO Rewrite this entire class.
    @Load
    String objectType;
    @Load
    String description;
    RequiredObjectType type;

    public RequiredObject(ConfigurationSection c) {
        load(c);
        type = getType(objectType);
    }

    public boolean isSatisfied(ConfigurationSection c, Arena a) {
        String name = getName();
        if (!c.contains(name))
            return false;
        switch (type) {
        case BOOLEAN:
            return isValidBoolean(name, c);
        case CLASS:
            return isValidClass(name, c);
        case DOUBLE:
            return isValidDouble(name, c);
        case INTEGER:
            return isValidInteger(name, c);
        case SPAWNPOINT:
            return isValidSpawnpoint(name, c, a);
        case STRING:
            return isValidString(name, c);
        case WAVE:
            return isValidWave(name, c);
        case LOCATION:
            return isValidLocation(name, c);
        default:
            return false;
        }
    }

    public boolean isValidLocation(String path, ConfigurationSection c) {
        return ConfigUtil.tryParseLocation(c.getString(path));
    }

    public boolean isValidBoolean(String path, ConfigurationSection c) {
        return (c.get(path) instanceof Boolean);
    }

    public boolean isValidDouble(String path, ConfigurationSection c) {
        return (c.get(path) instanceof Double);
    }

    public boolean isValidInteger(String path, ConfigurationSection c) {
        return (c.get(path) instanceof Integer);
    }

    public boolean isValidString(String path, ConfigurationSection c) {
        return (c.get(path) instanceof String);
    }

    public boolean isValidClass(String path, ConfigurationSection c) {
        return (BukkitArena.getClassManager().getClass(c.getString(path)) != null);
    }

    public boolean isValidWave(String path, ConfigurationSection c) {
        return (BukkitArena.getWaveManager().getWave(c.getString(path)) != null);
    }

    public boolean isValidSpawnpoint(String path, ConfigurationSection c, Arena a) {
        return (a.getSpawnpointManager().contains(c.getString(path)));
    }

    public static RequiredObjectType getType(String c) {
        return RequiredObjectType.valueOf(c.toUpperCase());
    }

    public enum RequiredObjectType {
        SPAWNPOINT, INTEGER, STRING, BOOLEAN, CLASS, WAVE, LOCATION, DOUBLE;

    }

    public String getDescription() {
        return description;
    }

}
