/**
 * 
 */
package com.bukkitarena.command;

import java.util.ArrayList;
import java.util.List;

import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;

/**
 * @author Nate Mortensen
 * 
 *         This is the class to be extended when a command is to be made. The
 *         follow methods must be implemented: getName() getSyntax()
 *         getPermissionNode() execute
 */
public abstract class ArenaCommand implements CommandExecutor {
    List<ArenaCommand> children;
    ArenaCommand parent;
    String syntax;

    public boolean onCommand(CommandSender sender, Command cmd, String commandLabel, String[] args) {
        execute(sender, commandLabel, args);
        return true;
    }

    protected boolean dispatchToChild(ArenaCommand c, CommandSender sender, String commandLabel, String[] args) {
        if (children.contains(c)) {
            c.execute(sender, commandLabel, args);
            c.setParent(this);
            return true;
        }
        return false;

    }

    public ArenaCommand getParent() {
        return parent;
    }

    public List<ArenaCommand> getChildren() {
        return children;
    }

    public int getMinimumArgs() {
        return 0;
    }

    public boolean isDefaultOp() {
        return true;
    }

    public List<String> getNames() {
        List<String> aliases = new ArrayList<String>();
        aliases.add(getName());
        return aliases;
    }

    public void setParent(ArenaCommand a) {
        parent = a;
    }

    public abstract String getName();

    /**
     * NOTE: This is for your part only. There is a hierarchy that will add in
     * the /a at the beginning. E.g., if you have the command "party time", and
     * it is used by time /a partytime <yes:no>", simple return "partytime
     * <yes:no>".
     * 
     * 
     * @return This command's portion of the syntax.
     */
    public abstract String getSyntax();

    public abstract void execute(CommandSender sender, String commandLabel, String[] args);

}
