/**
 * 
 */
package com.bukkitarena.configuration;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.bukkit.configuration.ConfigurationSection;

/**
 * @author Nate Mortensen
 * 
 * This is a simple manager designed for EasyConfiguration's.  The idea is that it uses a TreeMap with the names as keys, making it faster than a
 * List or most forms of storage.
 */
public class EasyManager<T extends EasyConfiguration> extends EasyConfiguration implements Iterable<T> {
    Map<String, T> map = new TreeMap<String, T>();

    Class<T> clazz;

    public EasyManager(Class<T> clazz, ConfigurationSection config) {
        load(config); //Called mostly for setting values, as well as allowing for this class to be extended with new fields added, which can be loaded.
        this.clazz = clazz;
        loadAllSubSections();

    }

    public T get(String name) {
        return map.get(name);
    }

    public boolean contains(String name) {
        return map.containsKey(name);
    }

    public Class<T> getType() {
        return clazz;
    }

    // All methods from Set, added in for ease of use of the manager. Becasuse
    // the manager isn't actually a set, it doesn't implement set.
    public boolean add(T t) {
        if (map.containsKey(t.getName()))
            if (map.get(t.getName()).equals(t))
                return false;
        map.put(t.getName(), t);
        return true;
    }

    public void loadAllSubSections() {
        for (String s : config.getKeys(false)) {
        	if (config.getConfigurationSection(s) != null) {
        		T ec = LoadingImpl.newInstance(clazz, config.getConfigurationSection(s));
        		add(ec);
        	}
        }
    }

    public boolean addAll(Collection<? extends T> c) {
        boolean returnVal = true;
        for (T t : c)
            if (!add(t))
                returnVal = false;
        return returnVal;

    }

    public void clear() {
        map.clear();

    }

    @SuppressWarnings("unchecked")
    public boolean contains(Object o) {
        return (o.getClass().isAssignableFrom(clazz) && map.containsKey(((T) o).getName()));

    }

    public boolean containsAll(Collection<?> c) {
        for (Object obj : c)
            if (!contains(obj))
                return false;
        return true;
    }

    public boolean isEmpty() {
        return (map.size() == 0);
    }

    public Iterator<T> iterator() {
        return map.values().iterator();
    }

    public boolean remove(Object o) {
        return (map.remove(o) != null);
    }

    public boolean removeAll(Collection<?> c) {
        boolean returnVal = true;
        for (Object t : c)
            if (!remove(t))
                returnVal = false;
        return returnVal;
    }

    public boolean retainAll(Collection<?> c) {
        boolean changed = false;
        for (Entry<String, T> entry : map.entrySet())
            if (!c.contains(entry.getValue()))
                if (map.remove(entry.getValue()) != null)
                    changed = true;
        return changed;
    }

    public int size() {
        return map.size();
    }

    public Object[] toArray() {
        return map.values().toArray();
    }

    public T[] toArray(T[] arg0) {
        return map.values().toArray(arg0);
    }

    public List<T> asList() {
        ArrayList<T> list = new ArrayList<T>();
        for (T t : this)
            list.add(t);
        return list;
    }

}
