package com.bukkitarena.configuration;

import java.lang.annotation.Annotation;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.bukkit.configuration.ConfigurationSection;

import com.bukkitarena.configuration.serialization.SerializationException;
import com.bukkitarena.configuration.serialization.SerializationManager;

public class LoadingImpl {

	@SuppressWarnings("unchecked")
	public static <T extends EasyConfiguration> T newInstance(Class<? extends T> clazz, ConfigurationSection c) {
		try {
			Constructor<T> constructor = (Constructor<T>) clazz.getConstructor(c.getClass());
			return constructor.newInstance(c);
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		return null;
	}
	public static void save(Object obj, ConfigurationSection config) {
		for (Field field : getFieldsWithAnnotation(obj, Load.class)) {
			String path = field.getName();
			if (config.contains(path)) {
				try {
					config.set(path, field.get(obj));
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public static void inheritFrom(Object obj1, Object obj2) {
		for (Field field : getFieldsWithAnnotation(obj1, Inherit.class)) {
			try {
				if (field.get(obj1) == null) {
					Field field2 = obj2.getClass().getField(field.getName());
					field2.setAccessible(true);
					if (field2.get(obj2) != null)
						field.set(obj1, field2.get(obj2));
				}
			} catch (SecurityException e) {
			} catch (NoSuchFieldException e) {
			} catch (IllegalAccessException e) {
			}
		}
	}
	public static void inherit(EasyConfiguration obj1) throws InvalidConfigurationException {
		for (Field field : getFieldsWithAnnotation(obj1, Inherit.class)) {
			String path = field.getAnnotation(Inherit.class).value();
			if (path.equals("")) {
				throw new DeveloperScrewedSomethingUpException("The field: \'"+field.getName()+"\' in class: \'"+obj1.getClass().getName()+"\' doesn't value assigned to its Inherit Annotation.");
			}
			ConfigurationSection c = obj1.getConfig().getParent().getConfigurationSection(path);
			if (c == null){
				throw new InvalidConfigurationException("Unable to inherit the value: \'"+ field.getName() + "\' from the section: \'"+obj1.getConfig().getCurrentPath()+"."+path+"\' as the section doesn't exist.");
			}
			try {
				if (field.get(obj1) == null) {
					field.set(obj1, c.get(field.getName()));
				}
			} catch (IllegalArgumentException e) {
				//Shouldn't happen
			} catch (IllegalAccessException e) {
				//Shouldn't happen
			}
		}
	}
	public static void inherit(Object obj1, ConfigurationSection section) {
		for (Field field : getFieldsWithAnnotation(obj1, Inherit.class)) {
			try {
				if (field.get(obj1) == null) {
					field.set(obj1, section.get(field.getName()));
				}
			} catch (IllegalArgumentException e) {
				//Shouldn't happen
			} catch (IllegalAccessException e) {
				//Shouldn't happen
			}
		}
	}

	public static List<Field> getFieldsWithAnnotation(Object obj, Class<? extends Annotation> clazz) {
		ArrayList<Field> fields = new ArrayList<Field>();
		for (Field field : obj.getClass().getDeclaredFields()) {
			try {
				field.setAccessible(true);
				if (field.isAnnotationPresent(clazz)) {
					fields.add(field);
				}
			} catch (SecurityException e) {
			}
		}
		return fields;
	}
	public static void handleFieldDeserialization(Field f, ConfigurationSection c, Object obj) {
		Class<?> clazz= f.getType();
		//If it's serializable and config contains a value.
		try {
			if (!c.contains(f.getName())) {
				System.out.println(f.getName());
				f.set(obj, null);
				return;
			}
			System.out.println("1- "+c.getCurrentPath()+"."+f.getName());
			System.out.println(c.get(f.getName()));

			if (SerializationManager.getInstance().canSerialize(clazz) ) { //Deserialize it.
				f.set(obj, SerializationManager.getInstance().deserialize(clazz, c, f.getName()));
				return;
			}
		} catch (IllegalArgumentException e) {
			throw new SerializationException(e);
		} catch (IllegalAccessException e) {
			throw new SerializationException(e);
		}

		throw new SerializationException("No valid serializer found for \'"+clazz.getName() +"\'.");
	}
	public static void handleFieldSerialization(Field f, ConfigurationSection c, Object obj) {
		Class<?> clazz = f.getType();
		if (SerializationManager.getInstance().canSerialize(clazz)) {
			SerializationManager.getInstance().serialize(clazz, c, f.getName(), c);
			return;
		}
		throw new SerializationException("Unable to serialize \'"+clazz.getName()+"\'.");
	}
	@SuppressWarnings({ "unchecked" })
	public static void load(Object obj, ConfigurationSection config) {
		for (Field field : getFieldsWithAnnotation(obj, Load.class)) {
			String path = field.getName();
			try {
				if (config.contains(path)) {
					//No need for a serializer for anything that extends EasyConfigurationSection.
					if (field.getType().isInstance(EasyConfiguration.class)) {
						field.set(obj, newInstance((Class<? extends EasyConfiguration>) field.getType(), config));
						continue;
					}
					handleFieldDeserialization(field, config, obj);
				} else {
					if (field.get(obj) != null) {
						//handleFieldSerialization(field, config, obj);
					} 
				}
			} catch (IllegalAccessException ex) {
				//Shouldn't happen.
			}

		}
	}
	//DEBUG - will be kept in the release, but this is debug code.
	public static void outputAllFields(Object obj) {
		for (Field f : obj.getClass().getDeclaredFields()) {
			f.setAccessible(true);
			try {
				System.out.println(f.getName() + " : "+f.get(obj));
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
