package com.bukkitarena.configuration.serialization;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map.Entry;

import org.bukkit.configuration.ConfigurationSection;

public class SerializationManager {
	
	static SerializationManager i;
	
	static {
		i = new SerializationManager();
		Serializer serializer = new NativeSerializer();
		i.registerSerializer(serializer, String.class);
		i.registerSerializer(serializer, Integer.class);
		i.registerSerializer(serializer, Boolean.class);
		i.registerSerializer(serializer, Byte.class);
		i.registerSerializer(serializer, Character.class);
		i.registerSerializer(serializer, Double.class);
		i.registerSerializer(serializer, Float.class);
		i.registerSerializer(serializer, Long.class);
		i.registerSerializer(serializer, Collection.class);
		i.registerSerializer(serializer, Short.class);
		i.registerSerializer(serializer, Array.class);
	}
	public static SerializationManager getInstance() {
		return i;
	}
	HashMap<Class<?>, Serializer> serializers = new HashMap<Class<?>, Serializer>();
	
	public void registerSerializer(Serializer serializer, Class<?> clazz) {
		serializers.put(clazz, serializer);
	}
	public Object deserialize(Class<?> clazz, ConfigurationSection c, String path) {
		if (serializers.containsKey(clazz)) {
			return serializers.get(clazz).deserialize(c, path);
		}
		for (Entry<Class<?>, Serializer> entry : serializers.entrySet()) {
			if (entry.getKey().isAssignableFrom(clazz)) {
				serializers.put(clazz, entry.getValue());
				return entry.getValue().deserialize(c, path);
			}
		}
		throw new SerializationException("No valid serializer found for \'"+clazz.getName() +"\'.");
	}
	public void serialize(Class<?> clazz, ConfigurationSection c, String path, Object obj) {
		if (serializers.containsKey(clazz)) {
			serializers.get(clazz).serialize(c, path, obj);
			return;
		}
		for (Entry<Class<?>, Serializer> entry : serializers.entrySet()) {
			if (entry.getKey().isAssignableFrom(clazz)) {
				serializers.put(clazz, entry.getValue());
				entry.getValue().serialize(c, path, obj);
				return;
			}
		}
		throw new SerializationException("No valid serializer found for \'"+clazz.getName() +"\'.");
	}
	public boolean canSerialize(Class<?> clazz) {
		if (serializers.containsKey(clazz)) {
			return true;
		}
		for (Entry<Class<?>, Serializer> entry : serializers.entrySet()) {
			if (entry.getKey().isAssignableFrom(clazz)) {
				serializers.put(clazz, entry.getValue());
				return true;
			}
		}
		return false;
	}

	public static boolean isNative(Class<?> clazz) {
		if (clazz.isPrimitive())
			return true;
		if (clazz.equals(String.class) || clazz.isAssignableFrom(Collection.class)) {
			return true;
		}
		return false;
	}
	

}
