/**
 * 
 */
package com.bukkitarena.entity;

import java.util.HashMap;

import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;

import com.bukkitarena.Msg;
import com.bukkitarena.arena.Arena;
import com.bukkitarena.playerclass.PlayerClass;
import com.bukkitarena.spawning.Spawnpoint;
import com.bukkitarena.team.TeamColor;
import com.bukkitarena.util.NamedObject;

/**
 * @author Nate Mortensen
 * 
 * 
 *         I put in a few methods for ease of access. All they do is return the
 *         method from Player. Yeah, the casting is a bit annoying. I prefer it
 *         to storing a different value for player, and ditching all inherited
 *         methods.
 */
public class ArenaPlayer extends ArenaEntity implements NamedObject {
    // Deaths are unique to a player, as currently mobs aren't saved from death
    // and brought back.
    int deaths = 0;
    // Mobs have no class.
    PlayerClass clazz;
    // The location said player is teleported to upon leaving.
    Location endLocation;

    TeamColor color;

    /**
     * @param p
     * @param a
     */
    public ArenaPlayer(Player p, Arena a, Location l) {
        super(p, a);
        arena = a;
        endLocation = l;
    }

    public String getName() {
        return ((Player) entity).getName();
    }

    public PlayerClass getPlayerClass() {
        return clazz;
    }

    // simply return getTeam()==null would return the opposite of what I want.
    public boolean hasTeam() {
        return getTeam() == null ? false : true;
    }

    public int getDeaths() {
        return deaths;
    }

    public int getKills() {
        return kills;
    }

    public Player getPlayer() {
        return (Player) entity;
    }

    public Arena getArena() {
        return arena;
    }

    public void sendMessage(String msg) {
        ((Player) entity).sendMessage(msg);
    }

    public TeamColor getTeam() {
        return color;
    }

    public void setTeam(TeamColor t) {
        color = t;
    }

    /**
     * Forcefully spawns a player at the specified point.
     * 
     * @param point
     *            Spawnpoint for them to spawn at.
     */
    public void spawnAt(Spawnpoint point) {
        ((Player) entity).setFoodLevel(20);
        entity.setHealth(20);
        ((Player) entity).setExp(0);
        entity.setFireTicks(0);
        entity.teleport(point);
    }

    public void tp(Location location) {
        entity.teleport(location);
    }

    private void setPlayerClass(PlayerClass c) {
        PlayerInventory inv = ((Player) entity).getInventory();
        inv.setHelmet(c.getHelmet());
        inv.setChestplate(c.getChestplate());
        inv.setBoots(c.getBoots());
        inv.setLeggings(c.getLeggings());
        for (ItemStack i : c.getInventory())
            inv.addItem(i);
    }

    public void selectPlayerClass(PlayerClass c) {
        if (c.getPermissionNode() == null || ((Player) entity).hasPermission(c.getPermissionNode())) {
            setPlayerClass(c);
            ((Player) entity).sendMessage(Msg.CLASS_SELECT_SUCCESS.replace("<class>", c.getName()));
            return;
        }
        ((Player) entity).sendMessage(Msg.CLASS_SELECT_FAIL_NOPERM.replace("<class>", c.getName()));

    }

    public HashMap<String, Object> getMetaData() {
        return metadata;
    }

    public Object getMetaDataValue(String key) {
        return metadata.get(key);
    }

    public void setMetaData(String key, Object value) {
        metadata.put(key, value);
    }

    /**
     * @return the score
     */
    public int getScore() {
        return score;
    }

    /**
     * @param score
     *            the score to set
     */
    public void setScore(int score) {
        this.score = score;
    }

    public void addScore(int val) {
        score += val;
    }

    public void removeScore(int val) {
        score -= val;
    }

    public Location getEndLocation() {
        return endLocation;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(String arg0) {
        return getName().compareTo(arg0);
    }

    public void sendHome() {
        entity.teleport(endLocation);
    }

}