/**
 * 
 */
package com.bukkitarena.event;

import java.lang.reflect.Method;

import org.bukkit.event.Cancellable;
import org.bukkit.event.Event;

import com.bukkitarena.arena.Arena;

/**
 * @author Nate Mortensen
 * 
 */
public abstract class ArenaEvent extends Event {
    private Arena arena;

    public ArenaEvent(Arena a) {
        arena = a;
    }

    public Arena getArena() {
        return arena;
    }

    /**
     * This method is a check to see if the event inherits the getHandlers
     * method, which means it ins't listenable.
     * 
     * @return
     */
    public boolean isListenable() {
        return isListenable(getClass());
    }

    @SuppressWarnings("unchecked")
    public static String getListenable(Class<? extends ArenaEvent> event) {
        if (isListenable(event)) {
            return event.getSimpleName();
        }
        Class<?> parent = event.getSuperclass();
        // Ensure the parent isn't null, and ensures the parent extends
        // ArenaEvent, but is NOT ArenaEvent.
        if (parent == null || (!parent.isAssignableFrom(ArenaEvent.class) && !parent.equals(ArenaEvent.class)))
            return "Broken Event, send report to evilmidget38.";
        return getListenable((Class<? extends ArenaEvent>) parent);

    }

    public static boolean isListenable(Class<? extends ArenaEvent> event) {
        for (Method m : event.getDeclaredMethods())
            if (m.getName().equalsIgnoreCase("gethandlers"))
                return true;
        return false;
    }

    public boolean isCancellable() {
        return getClass().isInstance(Cancellable.class);
    }

}
