/**
 * 
 */
package com.bukkitarena.event;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;

import org.bukkit.Bukkit;
import org.bukkit.event.Cancellable;
import org.bukkit.event.Event;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockEvent;
import org.bukkit.event.entity.EntityEvent;
import org.bukkit.event.inventory.InventoryEvent;
import org.bukkit.event.painting.PaintingEvent;
import org.bukkit.event.player.PlayerEvent;
import org.bukkit.event.vehicle.VehicleEvent;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.arena.Arena;
import com.bukkitarena.arenatype.ArenaType;

/**
 * @author Nate Mortensen
 * 
 */
public class EventManager {

    HashMap<ArenaType, ArrayList<ArenaEventExecutor>> internal = new HashMap<ArenaType, ArrayList<ArenaEventExecutor>>();

    @SuppressWarnings("unchecked")
    public void registerListeners(Listener l, ArenaType t) {
        for (Method m : l.getClass().getMethods())
            if (m.isAnnotationPresent(EventHandler.class))
                if (m.getParameterTypes().length == 1) {
                    if (m.getParameterTypes()[0].isAssignableFrom(Event.class)) {
                        Class<? extends Event> eclazz = (Class<? extends Event>) m.getParameterTypes()[0];
                        if (!internal.containsKey(t))
                            internal.put(t, new ArrayList<ArenaEventExecutor>());
                        internal.get(t).add(new ArenaEventExecutor(l, m, eclazz, t));
                    } else
                        throw new InvalidListenerException("Invalid arguments for method: " + m.getName() + " in class: " + l.getClass().getSimpleName() + " in type: " + t.toString());

                } else
                    throw new InvalidListenerException("Invalid arguments for method: " + m.getName() + " in class: " + l.getClass().getSimpleName() + " in type: " + t.toString());
    }

    public void clear() {
        internal.clear();
    }

    public void unregisterAll(ArenaType t) {
        internal.remove(t);

    }

    public void callEvent(ArenaEvent e) {
        ArenaEvent a = (ArenaEvent) e;
        callInternal(a, a.getArena());
        if (a.isCancellable())
            if (((Cancellable) a).isCancelled())
                return;
        Bukkit.getPluginManager().callEvent(e);
    }

    public void handleEvent(Event e) {
        Arena a = getArena(e);
        if (a != null)
            callInternal(e, a);
    }

    private void callInternal(Event e, Arena a) {
        for (ArenaEventExecutor ex : internal.get(a.getType())) {
            ex.execute(e);
        }
    }

    public static Arena getArena(Event e) {
        if (e instanceof PlayerEvent) {
            Arena a = BukkitArena.getArenaManager().getArena(((PlayerEvent) e).getPlayer());
            if (a != null) {
                return a;
            }

        } else if (e instanceof BlockEvent) {
            Arena a = BukkitArena.getArenaManager().getArena(((BlockEvent) e).getBlock().getLocation());
            if (a != null) {
                return a;
            }

        } else if (e instanceof VehicleEvent) {
            Arena a = BukkitArena.getArenaManager().getArena(((VehicleEvent) e).getVehicle().getLocation());
            if (a != null) {
                return a;
            }
        } else if (e instanceof EntityEvent) {
            Arena a = BukkitArena.getArenaManager().getArena(((EntityEvent) e).getEntity().getLocation());
            if (a != null) {
                return a;
            }
        } else if (e instanceof InventoryEvent) {
            Arena a = BukkitArena.getArenaManager().getArena(((InventoryEvent) e).getViewers().get(0));
            if (a != null) {
                return a;
            }
        } else if (e instanceof PaintingEvent) {
            Arena a = BukkitArena.getArenaManager().getArena(((PaintingEvent) e).getPainting().getLocation());
            if (a != null) {
                return a;
            }
        }
        return null;
    }
}
