/**
 * 
 */
package com.bukkitarena.listeners;

import org.bukkit.entity.LivingEntity;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.entity.CreatureSpawnEvent;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.entity.PlayerDeathEvent;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.entity.ArenaEntity;
import com.bukkitarena.entity.ArenaPlayer;
import com.bukkitarena.event.EventFactory;
import com.bukkitarena.event.entity.ArenaEntityDamageByEntityEvent;
import com.bukkitarena.event.entity.ArenaEntityDamageEvent;
import com.bukkitarena.event.entity.ArenaEntityDeathEvent;
import com.bukkitarena.event.player.ArenaPlayerDeathEvent;

/**
 * @author Nate Mortensen
 * 
 */
public class EntityListener extends BAListener {

    // Damage------------------------------------
    @EventHandler(priority = EventPriority.LOWEST)
    public void onEntityDamage(EntityDamageEvent e) {
        ArenaEntityDamageEvent event = toArenaEntityDamage(e);
        if (event == null)
            return;
        BukkitArena.getEventManager().callEvent(event);
        e.setDamage(event.getDamage());
        e.setCancelled(event.isCancelled());

    }

    private ArenaEntityDamageByEntityEvent toArenaEntityDamageByEntity(EntityDamageByEntityEvent e) {
        ArenaEntity victim = BukkitArena.getArenaManager().getArenaEntity((LivingEntity) e.getEntity());
        ArenaEntity damager = BukkitArena.getArenaManager().getArenaEntity((LivingEntity) e.getDamager());
        if (victim == null || damager == null)
            return null;
        ArenaEntityDamageByEntityEvent event = new ArenaEntityDamageByEntityEvent(victim, damager, e.getDamage());
        if (!(damager instanceof ArenaPlayer)) {
            event.setDamage(damager.getDamage());
        }
        return event;
    }

    private ArenaEntityDamageEvent toArenaEntityDamage(EntityDamageEvent e) {
        if (e instanceof EntityDamageByEntityEvent) {
            return toArenaEntityDamageByEntity((EntityDamageByEntityEvent) e);
        }
        ArenaEntity entity = BukkitArena.getArenaManager().getArenaEntity((LivingEntity) e);
        if (entity == null)
            return null;
        return new ArenaEntityDamageEvent(entity, e.getCause(), e.getDamage());
    }

    // Death-----------------------------------------------
    @EventHandler(priority = EventPriority.LOWEST)
    public void onEntityDeath(EntityDeathEvent e) {
        ArenaEntityDeathEvent event = toArenaEntityDeath(e);
        if (event == null)
            return;
        BukkitArena.getEventManager().callEvent(event);
        e.setDroppedExp(event.getDroppedExp());
    }

    private ArenaPlayerDeathEvent toArenaPlayerDeath(PlayerDeathEvent e) {
        ArenaPlayer player = BukkitArena.getArenaManager().getArenaPlayer(e.getEntity());
        if (player == null)
            return null;
        return new ArenaPlayerDeathEvent(player, e.getDrops(), e.getDroppedExp());
    }

    private ArenaEntityDeathEvent toArenaEntityDeath(EntityDeathEvent e) {
        if (e instanceof PlayerDeathEvent)
            return toArenaPlayerDeath((PlayerDeathEvent) e);
        ArenaEntity entity = BukkitArena.getArenaManager().getArenaEntity((LivingEntity) e.getEntity());
        if (entity == null)
            return null;
        return new ArenaEntityDeathEvent(entity, e.getDrops(), e.getDroppedExp());
    }

    @EventHandler(priority = EventPriority.HIGHEST)
    public void onEntitySpawn(CreatureSpawnEvent e) {
        if (BukkitArena.getArenaManager().getArena(e.getLocation()) != null)
            // Get out of my Arena! No natural mob spawning is allowed!
            e.setCancelled(true);
    }

}
