/**
 * 
 */
package com.bukkitarena.lobby;

import java.util.HashMap;

import org.bukkit.Location;
import org.bukkit.entity.Player;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.Msg;
import com.bukkitarena.arena.Arena;
import com.bukkitarena.arenatype.ArenaTypeDescriptionFile;
import com.bukkitarena.runnable.lobby.LobbyStartRunnable;

/**
 * @author Nate Mortensen
 * 
 */
public class Lobby {
    HashMap<Player, Location> players;

    LobbySettings settings;
    LobbyState state;
    Location loc;
    VoteManager<Arena> arenaVoting;
    VoteManager<ArenaTypeDescriptionFile> typeVoting;
    VoteManager<?> current;

    public Lobby(LobbySettings settings) {
        initialize(settings);
    }

    public void initialize(LobbySettings settings) {
        arenaVoting = new VoteManager<Arena>(this);
        this.settings = settings;
        for (String s : settings.getArenas()) {
            Arena a = BukkitArena.getArenaManager().getArena(s);
            if (a.isReady())
                arenaVoting.voteOptions.add(a);
        }
        current = arenaVoting;
        state = LobbyState.PREVOTE;
    }

    public void attemptPlayerJoin(Player p) {
        if (players.size() >= settings.getMaximumPlayers() && !(p.hasPermission("bukkitarena.lobby.maxoverride"))) {
            p.sendMessage(Msg.LOBBY_JOIN_FAIL_FULL);
        }
        p.sendMessage(Msg.LOBBY_JOIN);
        players.put(p, p.getLocation().clone());
    }

    public void startArena() {
        arenaVoting.winner.start(players, typeVoting.winner.getName());
    }

    public void reset() {
        players.clear();
        typeVoting = null;
        arenaVoting = null;
        initialize(settings);

    }

    public void start() {
        BukkitArena.getScheduler().schedule(new LobbyStartRunnable(this), settings.getTime() * 20);
        sendMessage(Msg.LOBBY_START.replaceAll("<second>", Integer.toString(settings.getTime())));
    }

    public void sendMessage(String msg) {
        for (Player p : current.votes.keySet()) {
            p.sendMessage(msg);
        }
    }

    public boolean isEmpty() {
        return players.isEmpty();
    }

    public void onVoteManagerEnd(VoteManager<?> m) {

    }

    public enum LobbyState {
        PREVOTE, VOTING_ARENA, VOTING_GAMETYPE, PREGAME;
    }

}
