/**
 * 
 */
package com.bukkitarena.lobby;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.bukkit.entity.Player;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.Msg;
import com.bukkitarena.runnable.lobby.VoteCheckRunnable;
import com.bukkitarena.util.NamedObject;
import com.bukkitarena.util.SortedNamedObjectList;

/**
 * @author Nate Mortensen
 * 
 */
public class VoteManager<T extends NamedObject> {

    private int timePassed = 0;
    private int time;

    HashMap<Player, T> votes = new HashMap<Player, T>();
    SortedNamedObjectList<T> voteOptions = new SortedNamedObjectList<T>();
    T winner;

    Lobby l;

    public VoteManager(Lobby l) {
        this.l = l;
        time = l.settings.time;
    }

    public boolean hasVoted(Player p) {
        return hasVoted(p.getName());

    }

    public boolean hasVoted(String s) {
        for (Player p : votes.keySet())
            if (p.getName().equalsIgnoreCase(s))
                return true;
        return false;
    }

    public void addPlayer(Player p) {
        votes.put(p, null);
        sendVotingOptions(p);
    }

    public void removePlayer(Player p) {
        votes.remove(p);
    }

    public void vote(Player p, String choice) {
        for (T t : voteOptions)
            if (t.toString().equalsIgnoreCase(choice)) {
                vote(p, t);
                return;
            }
        p.sendMessage(Msg.LOBBY_VOTE_FAIL_INVALID);
    }

    public void vote(Player p, T t) {
        if (votes.get(p).equals(t)) {
            p.sendMessage(Msg.LOBBY_VOTE_FAIL_SAME);
            return;
        }
        p.sendMessage(Msg.LOBBY_VOTE_SUCCESS);
        votes.put(p, t);
    }

    public void sendMessage(String msg) {
        for (Player p : votes.keySet())
            p.sendMessage(msg);
    }

    private boolean haveHalfVoted() {
        int total = 0;
        for (Player p : votes.keySet())
            if (hasVoted(p))
                total++;
        if (total >= votes.keySet().size() / 2)
            return true;
        return false;
    }

    private void startCheckRunnable() {
        int duration = time - timePassed;
        if (duration > 10) {
            duration = 10;
        }
        timePassed += duration;
        BukkitArena.getScheduler().schedule(new VoteCheckRunnable(this), duration * 20);
    }

    public void checkVoting() {
        if (timePassed > time) {
            sendMessage(Msg.LOBBY_VOTE_CHECK_SUCCESS_TIME);
            getWinner();
            return;
        }
        if (haveHalfVoted()) {
            sendMessage(Msg.LOBBY_VOTE_CHECK_SUCCESS_VOTES);
            getWinner();
            return;
        }
        sendMessage(Msg.LOBBY_VOTE_CHECK_FAIL_MOREVOTE);
        startCheckRunnable();
    }

    public HashMap<T, Integer> getCurrentStandings() {
        HashMap<T, Integer> map = new HashMap<T, Integer>();
        for (T t : voteOptions)
            map.put(t, 0);
        for (T t : votes.values())
            if (t != null)
                map.put(t, map.get(t) + 1);
        return map;

    }

    public void getWinner() {
        HashMap<T, Integer> standings = getCurrentStandings();
        List<Integer> values = (List<Integer>) standings.values();
        Collections.sort(values);
        if (values.get(0) == values.get(1)) {
            nextRoundVoting(values, standings);
            return;
        }
        for (T t : standings.keySet()) {
            if ((standings.get(t).equals(values.get(0)))) {
                winner = t;
                // Our purpose has been served.
                (l).onVoteManagerEnd(this);
                return;
            }
        }

    }

    public void nextRoundVoting(List<Integer> values, HashMap<T, Integer> standings) {
        sendMessage(Msg.LOBBY_VOTE_TIE);
        Iterator<Integer> iterator = values.iterator();
        while (iterator.hasNext())
            if (iterator.next() != values.get(0))
                iterator.remove();
        Iterator<T> it = voteOptions.iterator();
        while (it.hasNext())
            if (!values.contains(votes.get(it.next())))
                it.remove();
        for (Player p : votes.keySet())
            votes.put(p, null);
        sendMessage(Msg.LOBBY_VOTE_TIE);
        sendVotingOptions();
        timePassed = 0;
        time = l.settings.time;
        startCheckRunnable();
    }

    public void sendVotingOptions() {
        sendMessage(Msg.LOBBY_VOTE_START);
        HashMap<T, Integer> map = getCurrentStandings();
        for (T t : voteOptions)
            sendMessage("- " + t.toString() + "(" + Integer.toString(map.get(t)) + ")");
    }

    public void sendVotingOptions(Player p) {
        p.sendMessage(Msg.LOBBY_VOTE_START);
        HashMap<T, Integer> map = getCurrentStandings();
        for (T t : voteOptions)
            p.sendMessage("- " + t.toString() + "(" + Integer.toString(map.get(t)) + ")");
    }

}
