/**
 * 
 */
package com.bukkitarena.playerclass;

import java.util.ArrayList;
import java.util.List;

import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.inventory.ItemStack;

import com.bukkitarena.configuration.EasyConfiguration;
import com.bukkitarena.configuration.Load;
import com.bukkitarena.util.ComputeUtil;
import com.bukkitarena.util.ConfigUtil;
import com.bukkitarena.util.ItemUtil;

/**
 * @author Nate Mortensen
 * 
 *         If a permission node isn't defined, anyone can use it.
 * 
 */
public class PlayerClass extends EasyConfiguration {

    @Load
    int helm, chest, leg, boot;

    ItemStack helmet;
    ItemStack chestplate;
    ItemStack leggings;
    ItemStack boots;
    List<ItemStack> items;
    ArrayList<ItemStack> inventory = new ArrayList<ItemStack>();

    @Load
    String permissionNode = null;

    public PlayerClass(ConfigurationSection c) {
        load(c);
        ItemStack h = new ItemStack(helm, 1);
        ItemStack ch = new ItemStack(chest, 1);
        ItemStack l = new ItemStack(leg, 1);
        ItemStack b = new ItemStack(boot, 1);

        // Use various Util methods in order to load the inventory from the
        // config in hashmap form, convert it to ints, and then convert it to
        // itemstacks.
        ArrayList<ItemStack> inv = (ArrayList<ItemStack>) ItemUtil.intsToItemStacks(ComputeUtil.convertHashMapStringInt(ConfigUtil.<Integer> getHashMap(this.getConfig(), "inventory", Integer.class)));
        // Check if the items are valid armor pieces, excluding helmets.
        if (!ItemUtil.isChestPlate(ch)) {
            inv.add(ch.clone());
            ch = null;
        }
        if (!ItemUtil.isLegging(l)) {
            inv.add(l.clone());
            l = null;
        }
        if (ItemUtil.isBoots(b)) {
            inv.add(b.clone());
            b = null;
        }
        helmet = h;
        chestplate = ch;
        leggings = l;
        boots = b;
    }

    /**
     * @return the class's permission node. Can be null, indicates no node
     *         needed.
     */
    public String getPermissionNode() {
        return permissionNode;
    }

    /**
     * @return the helmet
     */
    public ItemStack getHelmet() {
        return helmet;
    }

    /**
     * @param helmet
     *            the helmet the player will wear.
     */
    public void setHelmet(ItemStack helmet) {
        this.helmet = helmet;
    }

    /**
     * @return the chestplate the player will wear.
     */
    public ItemStack getChestplate() {
        return chestplate;
    }

    /**
     * @param newval
     *            the permission node required to use this class.
     */
    public void setPermissionNode(String newval) {
        this.permissionNode = newval;
    }

    /**
     * @param chestplate
     *            the chestplate the player will wear.
     */
    public void setChestplate(ItemStack chestplate) {
        this.chestplate = chestplate;
    }

    /**
     * @return the leggings the player will wear.
     */
    public ItemStack getLeggings() {
        return leggings;
    }

    /**
     * @param leggings
     *            the leggings the player will wear.
     */
    public void setLeggings(ItemStack leggings) {
        this.leggings = leggings;
    }

    /**
     * @return the boots the player will wear.
     */
    public ItemStack getBoots() {
        return boots;
    }

    /**
     * @param boots
     *            the boots the player will wear.
     */
    public void setBoots(ItemStack boots) {
        this.boots = boots;
    }

    /**
     * @return the items that will be given to the player.
     */
    public ArrayList<ItemStack> getInventory() {
        return inventory;
    }

    /**
     * @param inventory
     *            the items the player will be given.
     */
    public void setInventory(ArrayList<ItemStack> inventory) {
        this.inventory = inventory;
    }

    /**
     * @param i
     *            the itemstack being added.
     */
    public void addItem(ItemStack i) {
        inventory.add(i);
    }

    /**
     * @param i
     *            the index of the item being removed.
     */
    public void removeItem(int i) {
        inventory.remove(i);
    }

}
