/**
 * 
 */
package com.bukkitarena.region;

import java.util.ArrayList;
import java.util.List;

import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Block;


import com.bukkitarena.util.ComputeUtil;

/**
 * @author Nate Mortensen
 * 
 */
public class CuboidRegion implements Region {
    Location p1;
    Location p2;

    public CuboidRegion(Location point1, Location point2) {
        // A nice and simple way to do the cuboid.
        double minx, miny, minz, maxx, maxy, maxz;
        minx = ComputeUtil.getMinimum(p1.getX(), p2.getX());
        miny = ComputeUtil.getMinimum(p1.getY(), p2.getY());
        minz = ComputeUtil.getMinimum(p1.getZ(), p2.getZ());
        maxx = ComputeUtil.getMaximum(p1.getX(), p2.getX());
        maxy = ComputeUtil.getMaximum(p1.getY(), p2.getZ());
        maxz = ComputeUtil.getMaximum(p1.getZ(), p2.getZ());
        p1 = new Location(point1.getWorld(), minx, miny, minz);
        p2 = new Location(point1.getWorld(), maxx, maxy, maxz);

    }

    public List<Block> getContainingBlocks() {
        World world = p1.getWorld();
        ArrayList<Block> blocks = new ArrayList<Block>();
        int xdist = (int) p1.getBlockX() - p2.getBlockX();
        int ydist = (int) p1.getBlockY() - p2.getBlockY();
        int zdist = (int) p1.getBlockZ() - p2.getBlockZ();
        for (int x = 0; x < xdist; x++)
            for (int y = 0; y < ydist; y++)
                for (int z = 0; z < zdist; z++)
                    blocks.add(new Location(world, x, y, z).getBlock());
        return blocks;
    }

    public boolean containsLocation(Location pos) {
        if (pos.getX() >= p1.getX() && pos.getY() >= p1.getY() && pos.getZ() >= p1.getZ())
            if (pos.getX() <= p2.getX() && pos.getY() <= p2.getY() && pos.getZ() <= p2.getZ())
                return true;
        return false;
    }

    public World getWorld() {
        return p1.getWorld();
    }

    /**
     * @return point 1
     */
    public Location getP1() {
        return p1;
    }

    /**
     * @param p1
     *            first point
     */
    public void setP1(Location p1) {
        this.p1 = p1;
    }

    /**
     * @return point 2
     */
    public Location getP2() {
        return p2;
    }

    /**
     * @param p2
     *            second point
     */
    public void setP2(Location p2) {
        this.p2 = p2;
    }



}
