/**
 * 
 */
package com.bukkitarena.spawning;

import java.util.HashSet;
import java.util.Set;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.EntityType;

import com.bukkitarena.team.TeamColor;

/**
 * @author Nate Mortensen
 * 
 */
public class Spawnpoint extends Location implements com.bukkitarena.util.NamedObject {
    /*
     * This is a list of entities that can be spawned here. Used when spawning
     * waves.
     */
    Set<EntityType> spawnable = new HashSet<EntityType>();
    /*
     * This is a team assigned to the spawn point. This is completely optional,
     * and is only used when trying to spawn players. If team is null it will be
     * ignored.
     */
    TeamColor team;

    String name;

    public Spawnpoint(World world, int x, int y, int z, float yaw, float pitch, String name) {
        super(world, x, y, z, yaw, pitch);
        this.name = name;
    }

    public Spawnpoint(ConfigurationSection c) {
        super(Bukkit.getWorld(c.getString("world")), c.getInt("x"), c.getInt("y"), c.getInt("z"));
        name = c.getString("name");
    }

    public boolean canSpawn(EntityType e) {
        return (spawnable.contains(e) ? true : false);
    }

    public boolean canSpawn(TeamColor t) {
        return team == null ? true : team.equals(t);
    }

    public void allowSpawn(EntityType e) {
        spawnable.add(e);
    }

    public void denySpawn(EntityType e) {
        if (spawnable.contains(e))
            spawnable.remove(e);
    }

    public TeamColor getTeam() {
        return team;
    }

    public void setTeam(TeamColor newval) {
        team = newval;
    }

    public int compareTo(String arg0) {
        return name.compareTo(arg0);
    }

    public String getName() {
        return name;
    }

}
