/**
 * 
 */
package com.bukkitarena.spawning;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

import org.bukkit.configuration.ConfigurationSection;

import com.bukkitarena.BukkitArena;
import com.bukkitarena.arena.Arena;
import com.bukkitarena.util.ConfigUtil;

/**
 * @author Nate Mortensen
 * 
 */
public class WaveSet {
    HashMap<Wave, Integer> waves = new HashMap<Wave, Integer>();

    public WaveSet(ConfigurationSection c) {
        loadWaves(c);
    }

    private void loadWaves(ConfigurationSection c) {
        HashMap<String, Integer> map = ConfigUtil.getHashMap(c, Integer.class);
        for (String s : map.keySet())
            waves.put(BukkitArena.getWaveManager().getWave(s), map.get(s));

    }

    public boolean isEmpty() {
        return (waves == null);
    }

    public Wave getWave(String name) {
        for (Wave wave : waves.keySet()) {
            if (wave.getName().equalsIgnoreCase(name))
                return wave;
        }
        return null;
    }

    public boolean hasWave(String name) {
        return (getWave(name) == null ? false : true);
    }

    public void addWave(Wave wave, int frequency) {
        waves.put(wave, frequency);
    }

    public Wave getNextWave() {
        HashMap<Wave, Integer> frequencies = getFrequencies();
        int random = new Random().nextInt(getTotal());
        ArrayList<Integer> f = new ArrayList<Integer>();
        for (Wave wave : waves.keySet()) {
            f.add(frequencies.get(wave));
        }
        Collections.sort(f);
        Collections.reverse(f);
        for (Integer freq : f)
            if (freq < random)
                for (Wave wave : frequencies.keySet())
                    if (frequencies.get(wave) == freq)
                        return wave;

        throw new IllegalStateException("Somehow no frequency was found.  Post your waves.yml and stacktrace on bukkitdev.");

    }

    public void spawnNextWave(Arena a) {
        spawnNextWave(a.getSpawnpointManager().getSpawnpoints());
    }

    public void spawnNextWave(List<Spawnpoint> spawn) {
        getNextWave().spawnWave(spawn);
    }

    /**
     * The integer frequency values for this map are VERY DIFFERENT from the
     * values of waves. The Integers in waves represent the values
     * 
     * 
     * @return
     */
    private HashMap<Wave, Integer> getFrequencies() {
        HashMap<Wave, Integer> map = new HashMap<Wave, Integer>();
        ArrayList<Integer> ints = new ArrayList<Integer>();
        ints.add(0);
        for (int i = 0; i < waves.size(); i++) {
            for (Wave wave : waves.keySet()) {
                int val = ints.get(i) + waves.get(wave);
                map.put(wave, val);
                ints.add(val);
            }
        }
        return map;
    }

    private int getTotal() {
        int total = 0;
        for (Wave wave : waves.keySet()) {
            total += waves.get(wave);
        }
        return total;
    }

}
