/**
 * 
 */
package com.bukkitarena.util;

import java.util.HashMap;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;

/**
 * @author Nate Mortensen
 * 
 */
public class ConfigUtil {

    /**
     * @param config
     *            FileConfiguration being modified.
     * @param path
     *            Path for the hashmap to use.
     * @param clazz
     *            The class of the type of value that the hashmap will be
     *            populated with.
     * @return A hashmap of the parameterized type.
     */
    @SuppressWarnings("unchecked")
    public static <T> HashMap<String, T> getHashMap(FileConfiguration config, String path, Class<T> clazz) {
        HashMap<String, T> hashmap = new HashMap<String, T>();
        if (!config.contains(path)) {
            return null;
        }
        ConfigurationSection section = config.getConfigurationSection(path);
        for (String key : section.getKeys(false)) {
            if (clazz.isInstance(section.get(key))) {
                hashmap.put(key, (T) section.get(key));
            }
        }

        return hashmap;
    }

    public static boolean tryParseLocation(String location) {
        try {
            parseLocation(location);
        } catch (Throwable ex) {
            return false;
        }
        return true;
    }

    public static Location parseLocation(String location) {
        String[] coords = location.split(",");
        return new Location(Bukkit.getServer().getWorld(coords[0]), Double.parseDouble(coords[1]), Double.parseDouble(coords[2]), Double.parseDouble(coords[3]));
    }

    public static String locationToString(Location location) {
        return location.getWorld().getName() + "," + Double.toString(location.getX()) + "," + Double.toString(location.getY()) + "," + Double.toString(location.getZ());
    }

    @SuppressWarnings("unchecked")
    public static <T> HashMap<String, T> getHashMap(ConfigurationSection config, String path, Class<T> clazz) {
        HashMap<String, T> hashmap = new HashMap<String, T>();
        if (!config.contains(path)) {
            return null;
        }
        ConfigurationSection section = config.getConfigurationSection(path);
        for (String key : section.getKeys(false)) {
            if (clazz.isInstance(section.get(key))) {
                hashmap.put(key, (T) section.get(key));
            }
        }

        return hashmap;
    }

    @SuppressWarnings("unchecked")
    public static <T> HashMap<String, T> getHashMap(ConfigurationSection config, Class<T> clazz) {
        HashMap<String, T> hashmap = new HashMap<String, T>();
        ConfigurationSection section = config;
        for (String key : section.getKeys(false)) {
            if (clazz.isInstance(section.get(key))) {
                hashmap.put(key, (T) section.get(key));
            }
        }

        return hashmap;
    }

    /**
     * @param config
     *            FileConfiguration being modified.
     * @param path
     *            The path of the section. No trailing "."'s.
     * @param hmap
     *            Hashmap that will be set.
     */
    public static void setHashMap(FileConfiguration config, String path, HashMap<String, ?> hmap) {
        for (String string : hmap.keySet()) {
            config.set(path + "." + string, hmap.get(string));
        }
    }

    /**
     * This method does not make it so a world is not saved with the location,
     * but rather, it overrides any world saved with the location. Useful for
     * world renames and what not.
     * 
     * @param location
     *            The String version of the location.
     * @param world
     * @return
     */
    public static Location parseLocation(String location, World world) {
        String[] coords = location.split(",");
        return new Location(world, Double.parseDouble(coords[1]), Double.parseDouble(coords[2]), Double.parseDouble(coords[3]));
    }

}
